<?php
/*
Plugin Name: Dark Visitors
Description: Protect your website from unwanted artificial agents.
Version: 0.2.0
Author URI: https://darkvisitors.com/
Author: Dark Visitors
*/

// Registration

function dark_visitors_register_settings() {
    add_option('dark_visitors_access_token', '');
    add_option('dark_visitors_is_block_ai_assistants_enabled', '0');
    add_option('dark_visitors_is_block_ai_data_scrapers_enabled', '0');
    add_option('dark_visitors_is_block_ai_search_crawlers_enabled', '0');
    add_option('dark_visitors_is_block_undocumented_ai_agents_enabled', '0');

    register_setting('dark-visitors-settings-group', 'dark_visitors_access_token');
    register_setting('dark-visitors-settings-group', 'dark_visitors_is_block_ai_assistants_enabled');
    register_setting('dark-visitors-settings-group', 'dark_visitors_is_block_ai_data_scrapers_enabled');
    register_setting('dark-visitors-settings-group', 'dark_visitors_is_block_ai_search_crawlers_enabled');
    register_setting('dark-visitors-settings-group', 'dark_visitors_is_block_undocumented_ai_agents_enabled');

    dark_visitors_clear_cache();
}

add_action('admin_init', 'dark_visitors_register_settings');

// Menu & Page

function dark_visitors_menu() {
    add_menu_page(
        // Page title
        'Dark Visitors',
        // Menu title
        'Dark Visitors',
        // Capability required to access the menu
        'manage_options',
        // Menu slug
        'dark-visitors',
        // Callback function to display the page
        'dark_visitors_page',
         // Menu icon
        'data:image/svg+xml;base64,' . base64_encode('<svg width="40px" height="52px" viewBox="0 0 40 52" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><g id="Page-1" stroke="none" stroke-width="1" fill="none" fill-rule="evenodd"><path d="M12,50 C12,51.1045695 11.1045695,52 10,52 C8.8954305,52 8,51.1045695 8,50 L8,48 C3.581722,48 0,44.418278 0,40 L0,24 C0,20.5099885 2.23480443,17.5419435 5.35143731,16.448841 C4.49775349,15.1779001 4,13.6470783 4,12 L4,8 C4,3.581722 7.581722,0 12,0 L28,0 C32.418278,0 36,3.581722 36,8 L36,12 C36,13.6470783 35.5022465,15.1779001 34.6489568,16.4502481 C37.7651956,17.5419435 40,20.5099885 40,24 L40,40 C40,44.418278 36.418278,48 32,48 L32,50 C32,51.1045695 31.1045695,52 30,52 C28.8954305,52 28,51.1045695 28,50 L28,47.999 L12,47.999 L12,50 Z M11.937,19.999 L8,20 C5.790861,20 4,21.790861 4,24 L4,40 C4,42.209139 5.790861,44 8,44 L32,44 C34.209139,44 36,42.209139 36,40 L36,24 C36,21.790861 34.209139,20 32,20 L28.062216,19.9997631 C28.041496,19.9999209 28.0207572,20 28,20 L12,20 L11.937,19.999 Z M28,4 L12,4 C9.790861,4 8,5.790861 8,8 L8,12 C8,14.1957503 9.76921937,15.9782772 11.9598802,15.999803 L28.04,15.999 L28.1996403,15.9951047 C30.3160315,15.8910789 32,14.1421954 32,12 L32,8 C32,5.790861 30.209139,4 28,4 Z M26,8 C27.1045695,8 28,8.8954305 28,10 C28,11.1045695 27.1045695,12 26,12 C24.8954305,12 24,11.1045695 24,10 C24,8.8954305 24.8954305,8 26,8 Z M14,8 C15.1045695,8 16,8.8954305 16,10 C16,11.1045695 15.1045695,12 14,12 C12.8954305,12 12,11.1045695 12,10 C12,8.8954305 12.8954305,8 14,8 Z" id="Combined-Shape" fill="#000000" fill-rule="nonzero"></path></g></svg>')
    );
}

function dark_visitors_page() {    
    ?>
    <style>
        .wrap {
            max-width: 800px;
            margin-left: auto;
            margin-right: auto;
        }

        table {
            width: 100%;
            border-collapse: collapse;
            text-align: left;
        }

        th, td {
            padding: 10px;
            border: 1px solid rgba(0, 0, 0, 0.1);
        }

        th {
            background-color: rgba(0, 0, 0, 0.05);
        }

        input[type="text"] {
            width: 100%;
        }

        td p {
            color: rgba(0, 0, 0, 0.5);
            margin-bottom: 0;
        }
    </style>
    <div class="wrap">
        <h1><?php echo esc_html(get_admin_page_title()); ?></h1>
        <p>Protect your website from unwanted artificial agents.</p>
        <p><a href="https://darkvisitors.com" target="_blank">Open Dark Visitors →</a></p>
        <h2>Configuration</h2>
        <form method="post" action="options.php" class="dark-visitors-form">
            <?php settings_fields('dark-visitors-settings-group'); ?>
            <table>
                <tr>
                    <th scope="row">
                        <label>Access Token</label>
                    </th>
                    <td>
                        <input type="text" id="dark_visitors_access_token" name="dark_visitors_access_token" value="<?php echo esc_attr(get_option('dark_visitors_access_token')); ?>" />
                        <p><a href="https://darkvisitors.com/sign-up" target="_blank">Sign up</a> and <a href="https://darkvisitors.com/projects" target="_blank">create a new project</a>. Navigate to the project's settings, copy its access token, and paste it here.</p>
                    </td>
                </tr>
                <tr>
                    <th scope="row">
                        <label>Block With Robots.txt</label>
                    </th>
                    <td>
                        <input type="checkbox" id="dark_visitors_is_block_ai_assistants_enabled" name="dark_visitors_is_block_ai_assistants_enabled" value="1" <?php checked('1', get_option('dark_visitors_is_block_ai_assistants_enabled')); ?> />
                        <label for="dark_visitors_is_block_ai_assistants_enabled">AI Assistants</label><br>
                        <input type="checkbox" id="dark_visitors_is_block_ai_data_scrapers_enabled" name="dark_visitors_is_block_ai_data_scrapers_enabled" value="1" <?php checked('1', get_option('dark_visitors_is_block_ai_data_scrapers_enabled')); ?> />
                        <label for="dark_visitors_is_block_ai_data_scrapers_enabled">AI Data Scrapers</label><br>
                        <input type="checkbox" id="dark_visitors_is_block_ai_search_crawlers_enabled" name="dark_visitors_is_block_ai_search_crawlers_enabled" value="1" <?php checked('1', get_option('dark_visitors_is_block_ai_search_crawlers_enabled')); ?> />
                        <label for="dark_visitors_is_block_ai_search_crawlers_enabled">AI Search Crawlers</label><br>
                        <input type="checkbox" id="dark_visitors_is_block_undocumented_ai_agents_enabled" name="dark_visitors_is_block_undocumented_ai_agents_enabled" value="1" <?php checked('1', get_option('dark_visitors_is_block_undocumented_ai_agents_enabled')); ?> />
                        <label for="dark_visitors_is_block_undocumented_ai_agents_enabled">Undocumented AI Agents</label><br>
                    </td>
                </tr>
            </table>
            <?php submit_button(); ?>
        </form>
    </div>
    <?php
}

add_action('admin_menu', 'dark_visitors_menu');

// Robots.txt

function dark_visitors_augment_robots_txt() {
    $now = time();
    $robots_txt_last_requested = get_option('dark_visitors_robots_txt_last_requested', 0);
    $one_day_in_seconds = 24 * 60 * 60;

    if ($now - $robots_txt_last_requested < $one_day_in_seconds) {
        $cached_robots_txt = get_option('dark_visitors_cached_robots_txt', '');
        dark_visitors_echo_robots_txt_block($cached_robots_txt);
    } else {
        $access_token = get_option('dark_visitors_access_token', '');
        $is_block_ai_assistants_enabled = get_option('dark_visitors_is_block_ai_assistants_enabled', '0') === '1';
        $is_block_ai_data_scrapers_enabled = get_option('dark_visitors_is_block_ai_data_scrapers_enabled', '0') === '1';
        $is_block_ai_search_crawlers_enabled = get_option('dark_visitors_is_block_ai_search_crawlers_enabled', '0') === '1';
        $is_block_undocumented_ai_agents_enabled = get_option('dark_visitors_is_block_undocumented_ai_agents_enabled', '0') === '1';

        $headers = array(
            'Content-Type' => 'application/json',
            'Authorization' => 'Bearer ' . $access_token
        );

        $body = array(
            'wordpress_plugin_version' => '0.2.0',
            'disallow' => '/',
            'agent_types' => array()
        );

        if ($is_block_ai_assistants_enabled) {
            array_push($body['agent_types'], 'AI Assistant');
        }

        if ($is_block_ai_data_scrapers_enabled) {
            array_push($body['agent_types'], 'AI Data Scraper');
        }

        if ($is_block_ai_search_crawlers_enabled) {
            array_push($body['agent_types'], 'AI Search Crawler');
        }

        if ($is_block_undocumented_ai_agents_enabled) {
            array_push($body['agent_types'], 'Undocumented AI Agent');
        }

        $response = wp_remote_post('https://api.darkvisitors.com/robots-txts', array(
            'headers' => $headers,
            'body' => json_encode($body),
            'blocking' => true
        ));

        if (!is_wp_error($response) && wp_remote_retrieve_response_code($response) >= 200 && wp_remote_retrieve_response_code($response) < 300 && isset($response['body'])) {
            $robots_txt = wp_remote_retrieve_body($response);

            update_option('dark_visitors_cached_robots_txt', $robots_txt);
            update_option('dark_visitors_robots_txt_last_requested', $now);

            dark_visitors_echo_robots_txt_block($robots_txt);
        }
    }
}

add_action('do_robots', 'dark_visitors_augment_robots_txt');

// Helpers

function dark_visitors_clear_cache() {
    delete_option('dark_visitors_cached_robots_txt');
    delete_option('dark_visitors_robots_txt_last_requested');
}

function dark_visitors_echo_robots_txt_block($robots_txt) {
    echo "

# START DARK VISITORS BLOCK
# ---------------------------
";

    echo $robots_txt;

    echo "
# ---------------------------
# END DARK VISITORS BLOCK

";
}